/*
   program       : various

   $Source: /home/yee/C/RCS/mutil.h,v $
   $Author: yee $
   $Date: 1999/02/10 16:38:05 $
   $Revision: 1.17 $

   Header file for math library routines.

   To Do :
     1) Need to change macro names to their capital letter counterparts.
     2) Change sinc(x) to a function.  As it stands, x is evaluated multiple
        times.

   Assumptions : none known

   Bugs : none known

   $Log: mutil.h,v $
   Revision 1.17  1999/02/10 16:38:05  yee
   *** empty log message ***

   Revision 1.16  1999/01/01 04:24:02  yee
   *** empty log message ***

   Copyright 1997 by Conway Yee.  All rights reserved.
 */

/*****************************************************************************/
/*****************************************************************************/

#ifndef MUTIL_H
#define MUTIL_H

  typedef struct complex {
    float r,i;
  } complex;
  typedef struct lcomplex {
    double r,i;
  } lcomplex;

#define MAX(a,b) (((a)>(b))?(a):(b))                   /* max & min macros */
#define MIN(a,b) (((a)<(b))?(a):(b))

#define NINT(x) ((int)floor((x)+0.5))                   /* rounding macros */
#define RINT(x) (floor((x)+0.5))

#define SIGN(x) (((x)<0.0)?-1:1) /* sign of a number */

#define BETWEEN(a,b,c) (((a)<=(b))&&((b)<=(c)))            /* is a<=b<=c ? */
#define BETWEEN_STRICT(a,b,c) (((a)<(b))&&((b)<(c)))        /* is a<b<c  ? */

#define TWO_PI 6.283185307179586476925287
#define PI_SQR 9.869604401089358618834491
#ifndef PI
#define PI   3.141592653589793238462643
#endif PI
#define PI_2   1.570796326794896619231322

#define cplus(a,b,c) {                   /* add two complex numbers */ \
            (a)->r = (b)->r + (c)->r;                                    \
            (a)->i = (b)->i + (c)->i;                                    \
          }

#define cminus(a,b,c) {             /* subtract two complex numbers */ \
            (a)->r = (b)->r - (c)->r;                                    \
            (a)->i = (b)->i - (c)->i;                                    \
          }

#define ctimes(a,b,c) {             /* multiply two complex numbers */ \
            double r,i;  /* dummy variables */                           \
            r = (b)->r * (c)->r - (b)->i * (c)->i;                       \
            i = (b)->i * (c)->r + (b)->r * (c)->i;                       \
            (a)->r = r;                                                  \
            (a)->i = i;                                                  \
          }

#define crtimes(a,b,c) {/* multiply complex (b) and real number (c) */ \
            double r,i;  /* dummy variables */                           \
            (a)->r = (b)->r * (c);                                       \
            (a)->i = (b)->i * (c);                                       \
          }

                          /* Can't use pointers since a,b,c may be the same */
#define dot(n,a,b,c) {  /* dot product of a=b*c */ \
            int j; /* counter */                     \
            for(j=0;j<(n);j++) (a)[j]=(b)[j]*(c)[j]; \
          }
                          /* Can't use pointers since a,b,c may be the same */
#define cdot(n,a,b,c) {  /* dot product (complex) of a=b*c */ \
            int j; /* counter */                                \
            for(j=0;j<(n);j++) ctimes((a)+j,(b)+j,(c)+j);       \
          }

#define crrdot(n,a,b,c) { /* dot project complex (b) and real number (c) \
                             (b) is assumed to have a zero imaginary       \
                              component */                                 \
            int j; /* counter */                                           \
            for(j=0;j<(n);j++) (a)[j].r = (b)[j].r * (c)[j];               \
          }

#define SINC(x) (fabs(x)<EPSILON?1.0:sin(x)/(x))

#define SQRT(x) pow((x),0.5)
#define SQR(x)  pow((x),2.0)

/* These macros implement recurrence formulae to calculate sin(theta)
   and cos(theta) from Numerical Recipes ed II cf 5.5.6 in which
          cos(t+delta) = cos(t)-[a*cos(t)+b*sin(t)]
          sin(t+delta) = sin(t)-[a*sin(t)-b*cos(t)]
               where a = 2*sin^2(delta/2)
                     b = sin(delta)

   a and b are coefficients that must be externally allocated. */

#define InitializeSinCosRecurrence(a,b,cos_t,sin_t,InitCos,InitSin,delta) {\
              { /* initial angle */                                           \
                 cos_t=InitCos;                                               \
                 sin_t=InitSin;                                               \
              } /* initial angle */                                           \
                                                                              \
              { /* recurrence formulae */                                     \
                 double dum=sin(0.5*delta); /* dummy variable */              \
                                                                              \
                 a=2*dum*dum;                                                 \
                 b=sin(delta);                                                \
              } /* recurrence formulae */                                     \
                                                                              \
           } /* InitializeSinCosRecurrence() */

#define UpdateSinCosRecurrence(a,b,sin_t,cos_t) {             \
             double dum; /* dummy variable */                    \
                                                                 \
             dum   = cos_t-(a*cos_t+b*sin_t);                    \
             sin_t = sin_t-(a*sin_t-b*cos_t);                    \
             cos_t = dum;                                        \
           }

#endif MUTIL_H
